# Whiteboard plotter hanging from strings
# derived from:
# -----------------------------------------------
# Plotter with x 2 y and pen up and pen down
# Virtual Machine file
# moves get set in Main
# usb port needs to be set in initInterfaces
# Nadya Peek August 2015
# -----------------------------------------------
# More info: 
# http://archive.fabacademy.org/archives/2016/fablabhrw/students/165/weekly09/weekly09.html
# Christoph Niess, April 2016

#------IMPORTS-------
from pygestalt import nodes
from pygestalt import interfaces
from pygestalt import machines
from pygestalt import functions
from pygestalt.machines import elements
from pygestalt.machines import kinematics
from pygestalt.machines import state
from pygestalt.utilities import notice
from pygestalt.publish import rpc	#remote procedure call dispatcher
import time
import io
from math import hypot


#------VIRTUAL MACHINE------
class virtualMachine(machines.virtualMachine):
	
	def initInterfaces(self):
		if self.providedInterface: self.fabnet = self.providedInterface		#providedInterface is defined in the virtualMachine class.
		else: self.fabnet = interfaces.gestaltInterface('FABNET', interfaces.serialInterface(baudRate = 115200, interfaceType = 'ftdi', portName = '/dev/tty.usbserial-FTYUDDO8'))
		
	def initControllers(self):
		self.lAxisNode = nodes.networkedGestaltNode('L Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence) # Left spool
		self.rAxisNode = nodes.networkedGestaltNode('R Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence) # Right spool
		self.pAxisNode = nodes.networkedGestaltNode('P Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence) # Pen lifter
		self.lrpNode = nodes.compoundNode(self.lAxisNode, self.rAxisNode, self.pAxisNode)

	def initCoordinates(self):
		self.position = state.coordinate(['mm', 'mm', 'mm'])
	
	def initKinematics(self):
		self.lAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(0.9), elements.leadscrew.forward(78.54), elements.invert.forward(True)])
		self.rAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(0.9), elements.leadscrew.forward(78.54), elements.invert.forward(False)])
		self.pAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(0.8), elements.invert.forward(True)])
		self.stageKinematics = kinematics.cableBot2D(leftSpoolX = -110, leftSpoolY = 670, rightSpoolX = 640, rightSpoolY = 670)	

	def initFunctions(self):
		self.move = functions.move(virtualMachine = self, virtualNode = self.lrpNode, axes = [self.lAxis, self.rAxis, self.pAxis], kinematics = self.stageKinematics, machinePosition = self.position,planner = 'null')
		self.jog = functions.jog(self.move)	#an incremental wrapper for the move function
		pass
		
	def initLast(self):
		self.lAxisNode.setMotorCurrent(0.7)
		self.rAxisNode.setMotorCurrent(0.7)
		self.pAxisNode.setMotorCurrent(0.2)
		self.lAxisNode.setVelocityRequest(2)
		self.rAxisNode.setVelocityRequest(2)
		self.pAxisNode.setVelocityRequest(5)
		pass
	
	def publish(self):
		#self.publisher.addNodes(self.machineControl)
		pass
	
	def getPosition(self):
		return {'position':self.position.future()}
	
	def setPosition(self, position  = [None]):
		self.position.future.set(position)

	def setSpindleSpeed(self, speedFraction):
		#self.machineControl.pwmRequest(speedFraction)
		pass

#------IF RUN DIRECTLY FROM TERMINAL------
if __name__ == '__main__':
	# The persistence file remembers the node you set. It'll generate the first time you run the
	# file. If you are hooking up a new node, delete the previous persistence file.
	machina = virtualMachine(persistenceFile = "whiteskater.vmp")	
	
	# Definition of the machine dimensions
	pendown = 5 # jog height for pen

	#emergency move to get the pen back up after trouble
	#machina.move([0,0,-5],0)
	#time.sleep(5.0)
	#import sys
	#sys.exit(0)

	imagefile = open('lift-demo2.csv', 'r')

	for line in imagefile.readlines():
		coor = [0,0,0]
		xyz = line.split(',')
		if len(xyz) == 1:
			print "Starting wait time for Z-Axis"
			time.sleep(5)
			print "End of wait time for Z-Axis"
		else:
			coor[0] = float(xyz[0]) / 10
			coor[1] = float(xyz[1]) / 10
			if len(xyz) == 2:
				coor[2] = pendown
			elif float(xyz[2]) != 0:
				coor[2]=pendown
			else:
				coor[2] = 0
			machina.move(coor,0)
			statusL = machina.lAxisNode.spinStatusRequest()
			statusR = machina.rAxisNode.spinStatusRequest()
			statusP = machina.pAxisNode.spinStatusRequest()
			todo = statusL['stepsRemaining'] + statusR['stepsRemaining'] + statusP['stepsRemaining']
			# This checks to see if the move is done.
			while todo > 0:
				time.sleep(0.001)
				statusL = machina.lAxisNode.spinStatusRequest()
				statusR = machina.rAxisNode.spinStatusRequest()
				statusP = machina.pAxisNode.spinStatusRequest()
				todo = statusL['stepsRemaining'] + statusR['stepsRemaining'] + statusP['stepsRemaining']
